@@ -6,11 +6,9 @@
 #include "pysidelogging_p.h"
 #include "pysideproperty.h"
 #include "pysideproperty_p.h"
-#include "pysidecleanup.h"
 #include "pyside_p.h"
 #include "dynamicqmetaobject.h"
 #include "pysidemetafunction_p.h"
-#include "pysidestaticstrings.h"
 
 #include <autodecref.h>
 #include <basewrapper.h>
@@ -21,37 +19,31 @@
 #include <sbkstaticstrings.h>
 #include <sbkerrors.h>
 
+#include <QtCore/QCoreApplication>
 #include <QtCore/QByteArrayView>
 #include <QtCore/QDebug>
 #include <QtCore/QHash>
 #include <QtCore/QScopedPointer>
+#include <QtCore/QTimerEvent>
 
-#include <algorithm>
-#include <limits>
 #include <memory>
 
+using namespace Qt::StringLiterals;
+
 #if QSLOT_CODE != 1 || QSIGNAL_CODE != 2
 #error QSLOT_CODE and/or QSIGNAL_CODE changed! change the hardcoded stuff to the correct value!
 #endif
-#define PYSIDE_SLOT '1'
-#define PYSIDE_SIGNAL '2'
-#include "globalreceiverv2.h"
 
-namespace {
-    static PyObject *metaObjectAttr = nullptr;
+static PyObject *metaObjectAttr = nullptr;
 
-    static PyObject *parseArguments(const QList< QByteArray >& paramTypes, void **args);
-    static bool emitShortCircuitSignal(QObject *source, int signalIndex, PyObject *args);
-
-    static void destroyMetaObject(PyObject *obj)
-    {
-        void *ptr = PyCapsule_GetPointer(obj, nullptr);
-        auto meta = reinterpret_cast<PySide::MetaObjectBuilder *>(ptr);
-        SbkObject *wrapper = Shiboken::BindingManager::instance().retrieveWrapper(meta);
-        if (wrapper)
-            Shiboken::BindingManager::instance().releaseWrapper(wrapper);
-        delete meta;
-    }
+static void destroyMetaObject(PyObject *obj)
+{
+    void *ptr = PyCapsule_GetPointer(obj, nullptr);
+    auto *meta = reinterpret_cast<PySide::MetaObjectBuilder *>(ptr);
+    SbkObject *wrapper = Shiboken::BindingManager::instance().retrieveWrapper(meta);
+    if (wrapper)
+        Shiboken::BindingManager::instance().releaseWrapper(wrapper);
+    delete meta;
 }
 
 static const char *metaCallName(QMetaObject::Call call)
@@ -72,6 +64,41 @@ static const char *metaCallName(QMetaObject::Call call)
     return it != mapping.constEnd() ? it.value() : "<Unknown>";
 }
 
+static QByteArray methodSignature(const QMetaMethod &method)
+{
+    QByteArray result;
+    if (const auto *t = method.typeName()) {
+        result += t;
+        result += ' ';
+    }
+    result += method.methodSignature();
+    return result;
+}
+
+static QByteArray msgCannotConvertParameter(const QByteArray &parameterTypeName,
+                                            const QByteArray &signature, qsizetype p)
+{
+    return "Cannot call meta function \""_ba + signature
+           + "\" because parameter " + QByteArray::number(p) + " of type \""_ba
+           + parameterTypeName + "\" cannot be converted."_ba;
+}
+
+static inline QByteArray msgCannotConvertParameter(QMetaMethod method, qsizetype p)
+{
+    return msgCannotConvertParameter(method.parameterTypeName(p),
+                                     methodSignature(method), p);
+}
+
+static QByteArray msgCannotConvertReturn(const QByteArray &signature)
+{
+    return "The return value of \""_ba + signature + "\" cannot be converted."_ba;
+}
+
+static inline QByteArray msgCannotConvertReturn(QMetaMethod method)
+{
+    return msgCannotConvertReturn(methodSignature(method));
+}
+
 namespace PySide {
 
 PyObjectWrapper::PyObjectWrapper()
@@ -129,6 +156,14 @@ PyObjectWrapper::operator PyObject *() const
     return m_me;
 }
 
+
+int PyObjectWrapper::toInt() const
+{
+    // hold the GIL
+    Shiboken::GilState state;
+    return Shiboken::Enum::check(m_me) ? Shiboken::Enum::getValue(m_me) : -1;
+}
+
 QDataStream &operator<<(QDataStream &out, const PyObjectWrapper &myObj)
 {
     if (Py_IsInitialized() == 0) {
@@ -149,8 +184,8 @@ QDataStream &operator<<(QDataStream &out, const PyObjectWrapper &myObj)
         const char *buff = nullptr;
         Py_ssize_t size  = 0;
         if (PyBytes_Check(repr.object())) {
-            buff = PyBytes_AS_STRING(repr.object());
-            size = PyBytes_GET_SIZE(repr.object());
+            buff = PyBytes_AsString(repr.object());
+            size = PyBytes_Size(repr.object());
         } else if (Shiboken::String::check(repr.object())) {
             buff = Shiboken::String::toCString(repr.object());
             size = Shiboken::String::len(repr.object());
@@ -190,27 +225,10 @@ QDataStream &operator>>(QDataStream &in, PyObjectWrapper &myObj)
 
 using namespace PySide;
 
-struct SignalManager::SignalManagerPrivate
+struct SignalManagerPrivate
 {
-    GlobalReceiverV2MapPtr m_globalReceivers;
     static SignalManager::QmlMetaCallErrorHandler m_qmlMetaCallErrorHandler;
 
-    SignalManagerPrivate() : m_globalReceivers(new GlobalReceiverV2Map{})
-    {
-    }
-
-    ~SignalManagerPrivate()
-    {
-        if (!m_globalReceivers.isNull()) {
-            // Delete receivers by always retrieving the current first element, because deleting a
-            // receiver can indirectly delete another one, and if we use qDeleteAll, that could
-            // cause either a double delete, or iterator invalidation, and thus undefined behavior.
-            while (!m_globalReceivers->isEmpty())
-                delete *m_globalReceivers->cbegin();
-            Q_ASSERT(m_globalReceivers->isEmpty());
-        }
-    }
-
     static void handleMetaCallError(QObject *object, int *result);
     static int qtPropertyMetacall(QObject *object, QMetaObject::Call call,
                                   int id, void **args);
@@ -218,12 +236,7 @@ struct SignalManager::SignalManagerPrivate
 };
 
 SignalManager::QmlMetaCallErrorHandler
-    SignalManager::SignalManagerPrivate::m_qmlMetaCallErrorHandler = nullptr;
-
-static void clearSignalManager()
-{
-    PySide::SignalManager::instance().clear();
-}
+    SignalManagerPrivate::m_qmlMetaCallErrorHandler = nullptr;
 
 static void PyObject_PythonToCpp_PyObject_PTR(PyObject *pyIn, void *cppOut)
 {
@@ -235,19 +248,21 @@ static PythonToCppFunc is_PyObject_PythonToCpp_PyObject_PTR_Convertible(PyObject
 }
 static PyObject *PyObject_PTR_CppToPython_PyObject(const void *cppIn)
 {
-    auto pyOut = reinterpret_cast<PyObject *>(const_cast<void *>(cppIn));
+    auto *pyOut = reinterpret_cast<PyObject *>(const_cast<void *>(cppIn));
     if (pyOut)
         Py_INCREF(pyOut);
     return pyOut;
 }
 
-SignalManager::SignalManager() : m_d(new SignalManagerPrivate)
+void SignalManager::init()
 {
     // Register Qt primitive typedefs used on signals.
     using namespace Shiboken;
 
     // Register PyObject type to use in queued signal and slot connections
     qRegisterMetaType<PyObjectWrapper>("PyObject");
+    // Register QVariant(enum) conversion to QVariant(int)
+    QMetaType::registerConverter<PyObjectWrapper, int>(&PyObjectWrapper::toInt);
 
     SbkConverter *converter = Shiboken::Conversions::createConverter(&PyBaseObject_Type, nullptr);
     Shiboken::Conversions::setCppPointerToPythonFunction(converter, PyObject_PTR_CppToPython_PyObject);
@@ -257,82 +272,15 @@ SignalManager::SignalManager() : m_d(new SignalManagerPrivate)
     Shiboken::Conversions::registerConverterName(converter, "PyObjectWrapper");
     Shiboken::Conversions::registerConverterName(converter, "PySide::PyObjectWrapper");
 
-    PySide::registerCleanupFunction(clearSignalManager);
-
     if (!metaObjectAttr)
         metaObjectAttr = Shiboken::String::fromCString("__METAOBJECT__");
 }
 
-void SignalManager::clear()
-{
-    delete m_d;
-    m_d = new SignalManagerPrivate();
-}
-
-SignalManager::~SignalManager()
-{
-    delete m_d;
-}
-
-SignalManager &SignalManager::instance()
-{
-    static SignalManager me;
-    return me;
-}
-
 void SignalManager::setQmlMetaCallErrorHandler(QmlMetaCallErrorHandler handler)
 {
     SignalManagerPrivate::m_qmlMetaCallErrorHandler = handler;
 }
 
-QObject *SignalManager::globalReceiver(QObject *sender, PyObject *callback)
-{
-    GlobalReceiverV2MapPtr globalReceivers = m_d->m_globalReceivers;
-    GlobalReceiverKey key = GlobalReceiverV2::key(callback);
-    GlobalReceiverV2 *gr = nullptr;
-    auto it = globalReceivers->find(key);
-    if (it == globalReceivers->end()) {
-        gr = new GlobalReceiverV2(callback, globalReceivers);
-        globalReceivers->insert(key, gr);
-        if (sender) {
-            gr->incRef(sender); // create a link reference
-            gr->decRef(); // remove extra reference
-        }
-    } else {
-        gr = it.value();
-        if (sender)
-            gr->incRef(sender);
-    }
-
-    return reinterpret_cast<QObject *>(gr);
-}
-
-int SignalManager::countConnectionsWith(const QObject *object)
-{
-    int count = 0;
-    for (GlobalReceiverV2Map::const_iterator it = m_d->m_globalReceivers->cbegin(), end = m_d->m_globalReceivers->cend(); it != end; ++it) {
-        if (it.value()->refCount(object))
-            count++;
-    }
-    return count;
-}
-
-void SignalManager::notifyGlobalReceiver(QObject *receiver)
-{
-    reinterpret_cast<GlobalReceiverV2 *>(receiver)->notify();
-}
-
-void SignalManager::releaseGlobalReceiver(const QObject *source, QObject *receiver)
-{
-    auto gr = reinterpret_cast<GlobalReceiverV2 *>(receiver);
-    gr->decRef(source);
-}
-
-int SignalManager::globalReceiverSlotIndex(QObject *receiver, const char *signature) const
-{
-    return reinterpret_cast<GlobalReceiverV2 *>(receiver)->addSlot(signature);
-}
-
 bool SignalManager::emitSignal(QObject *source, const char *signal, PyObject *args)
 {
     if (!Signal::checkQtSignal(signal))
@@ -340,20 +288,11 @@ bool SignalManager::emitSignal(QObject *source, const char *signal, PyObject *ar
     signal++;
 
     int signalIndex = source->metaObject()->indexOfSignal(signal);
-    if (signalIndex != -1) {
-        // cryptic but works!
-        // if the signature doesn't have a '(' it's a shor circuited signal, i.e. std::find
-        // returned the string null terminator.
-        bool isShortCircuit = !*std::find(signal, signal + std::strlen(signal), '(');
-        return isShortCircuit
-            ? emitShortCircuitSignal(source, signalIndex, args)
-            : MetaFunction::call(source, signalIndex, args);
-    }
-    return false;
+    return signalIndex != -1 && MetaFunction::call(source, signalIndex, args);
 }
 
 // Handle errors from meta calls. Requires GIL and PyErr_Occurred()
-void SignalManager::SignalManagerPrivate::handleMetaCallError(QObject *object, int *result)
+void SignalManagerPrivate::handleMetaCallError(QObject *object, int *result)
 {
     // Bubbles Python exceptions up to the Javascript engine, if called from one
     if (m_qmlMetaCallErrorHandler) {
@@ -361,7 +300,11 @@ void SignalManager::SignalManagerPrivate::handleMetaCallError(QObject *object, i
         if (idOpt.has_value())
             *result = idOpt.value();
     }
+    SignalManager::handleMetaCallError();
+}
 
+void SignalManager::handleMetaCallError()
+{
     const int reclimit = Py_GetRecursionLimit();
     // Inspired by Python's errors.c: PyErr_GivenExceptionMatches() function.
     // Temporarily bump the recursion limit, so that PyErr_Print will not raise a recursion
@@ -373,9 +316,9 @@ void SignalManager::SignalManagerPrivate::handleMetaCallError(QObject *object, i
 }
 
 // Handler for QMetaObject::ReadProperty/WriteProperty/ResetProperty:
-int SignalManager::SignalManagerPrivate::qtPropertyMetacall(QObject *object,
-                                                            QMetaObject::Call call,
-                                                            int id, void **args)
+int SignalManagerPrivate::qtPropertyMetacall(QObject *object,
+                                             QMetaObject::Call call,
+                                             int id, void **args)
 {
     const QMetaObject *metaObject = object->metaObject();
     int result = id - metaObject->propertyCount();
@@ -404,7 +347,9 @@ int SignalManager::SignalManagerPrivate::qtPropertyMetacall(QObject *object,
     if (PyErr_Occurred()) {
         // PYSIDE-2160: An unknown type was reported. Indicated by StopIteration.
         if (PyErr_ExceptionMatches(PyExc_StopIteration)) {
-            PyObject *excType, *excValue, *excTraceback;
+            PyObject *excType{};
+            PyObject *excValue{};
+            PyObject *excTraceback{};
             PyErr_Fetch(&excType, &excValue, &excTraceback);
             bool ign = call == QMetaObject::WriteProperty;
             PyErr_WarnFormat(PyExc_RuntimeWarning, 0,
@@ -412,7 +357,7 @@ int SignalManager::SignalManagerPrivate::qtPropertyMetacall(QObject *object,
                     : "Unknown property type '%s' of QObject '%s' used in fget with %R",
                 pp->d->typeName.constData(), metaObject->className(), excValue);
             if (PyErr_Occurred())
-                Shiboken::Errors::storeError();
+                Shiboken::Errors::storeErrorOrPrint();
             Py_DECREF(excType);
             Py_DECREF(excValue);
             Py_XDECREF(excTraceback);
@@ -428,8 +373,8 @@ int SignalManager::SignalManagerPrivate::qtPropertyMetacall(QObject *object,
 }
 
 // Handler for QMetaObject::InvokeMetaMethod
-int SignalManager::SignalManagerPrivate::qtMethodMetacall(QObject *object,
-                                                          int id, void **args)
+int SignalManagerPrivate::qtMethodMetacall(QObject *object,
+                                           int id, void **args)
 {
     const QMetaObject *metaObject = object->metaObject();
     const QMetaMethod method = metaObject->method(id);
@@ -455,14 +400,14 @@ int SignalManager::SignalManagerPrivate::qtMethodMetacall(QObject *object,
             PyErr_Format(PyExc_AttributeError, "Slot '%s::%s' not found.",
                          metaObject->className(), method.methodSignature().constData());
         } else {
-            SignalManager::callPythonMetaMethod(method, args, pyMethod, false);
+            SignalManager::callPythonMetaMethod(method, args, pyMethod);
         }
     }
     // WARNING Isn't safe to call any metaObject and/or object methods beyond this point
     //         because the object can be deleted inside the called slot.
 
-    if (gil.get() == nullptr)
-        gil.reset(new Shiboken::GilState);
+    if (gil == nullptr)
+        gil = std::make_unique<Shiboken::GilState>();
 
     if (PyErr_Occurred())
         handleMetaCallError(object, &result);
@@ -493,46 +438,122 @@ int SignalManager::qt_metacall(QObject *object, QMetaObject::Call call, int id,
                 << metaCallName(call) << " #" << id << ' '  << object;
             id -= object->metaObject()->methodCount();
             break;
+#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+        case QMetaObject::ConstructInPlace:
+            break;
+#endif
     }
     return id;
 }
 
-int SignalManager::callPythonMetaMethod(const QMetaMethod &method, void **args, PyObject *pyMethod, bool isShortCuit)
+// Helper for calling a Python pyCallable matching a Qt signal / slot.
+enum CallResult : int
 {
-    Q_ASSERT(pyMethod);
+    CallOk,
+    CallOtherError, // Python error set
+    CallReturnValueError,
+    CallArgumentError // Argument (return - CallArgumentError) caused an error
+};
+
+static inline bool isNonVoidReturn(const char *returnType)
+{
+    return returnType != nullptr && returnType[0] != 0 && std::strcmp("void", returnType) != 0;
+}
+
+static int callPythonMetaMethodHelper(const QByteArrayList &paramTypes,
+                                      const char *returnType /* = nullptr */,
+                                      void **args, PyObject *pyCallable)
+{
+    const qsizetype argsSize = paramTypes.size();
+    Shiboken::AutoDecRef preparedArgs(PyTuple_New(argsSize));
+
+    for (qsizetype i = 0; i < argsSize; ++i) {
+        void *data = args[i + 1];
+        auto param = paramTypes.at(i);
+        Shiboken::Conversions::SpecificConverter converter(param.constData());
+        if (!converter.isValid())
+            return CallResult::CallArgumentError + int(i);
+        PyTuple_SetItem(preparedArgs, i, converter.toPython(data));
+    }
+
+    QScopedPointer<Shiboken::Conversions::SpecificConverter> retConverter;
+    if (isNonVoidReturn(returnType)) {
+        retConverter.reset(new Shiboken::Conversions::SpecificConverter(returnType));
+        if (!retConverter->isValid())
+            return CallResult::CallReturnValueError;
+    }
+
+    Shiboken::AutoDecRef retval(PyObject_CallObject(pyCallable, preparedArgs.object()));
+    if (PyErr_Occurred() != nullptr || retval.isNull())
+        return CallResult::CallOtherError;
+
+    if (retval != Py_None && !retConverter.isNull())
+        retConverter->toCpp(retval, args[0]);
+    return CallResult::CallOk;
+}
+
+int SignalManager::callPythonMetaMethod(QMetaMethod method, void **args,
+                                        PyObject *callable)
+{
+    Q_ASSERT(callable);
 
     Shiboken::GilState gil;
-    PyObject *pyArguments = nullptr;
-
-    if (isShortCuit){
-        pyArguments = reinterpret_cast<PyObject *>(args[1]);
-    } else {
-        pyArguments = parseArguments(method.parameterTypes(), args);
+    int callResult = callPythonMetaMethodHelper(method.parameterTypes(),
+                                                method.typeName(), args, callable);
+    switch (callResult) {
+    case CallOk:
+        return 0;
+    case CallOtherError:
+        return -1;
+    case CallReturnValueError:
+        PyErr_SetString(PyExc_RuntimeError, msgCannotConvertReturn(method).constData());
+        return -1;
+    default: { // CallArgumentError + n
+        const int arg = callResult - CallArgumentError;
+        PyErr_SetString(PyExc_TypeError, msgCannotConvertParameter(method, arg).constData());
+        return -1;
     }
-
-    if (pyArguments) {
-        QScopedPointer<Shiboken::Conversions::SpecificConverter> retConverter;
-        const char *returnType = method.typeName();
-        if (returnType && std::strcmp("", returnType) && std::strcmp("void", returnType)) {
-            retConverter.reset(new Shiboken::Conversions::SpecificConverter(returnType));
-            if (!retConverter->isValid()) {
-                PyErr_Format(PyExc_RuntimeError, "Can't find converter for '%s' to call Python meta method.", returnType);
-                return -1;
-            }
-        }
-
-        Shiboken::AutoDecRef retval(PyObject_CallObject(pyMethod, pyArguments));
-
-        if (!isShortCuit && pyArguments){
-            Py_DECREF(pyArguments);
-        }
-
-        if (!retval.isNull() && retval != Py_None && !PyErr_Occurred() && retConverter) {
-            retConverter->toCpp(retval, args[0]);
-        }
     }
+    return 0;
+}
 
-    return -1;
+static QByteArray signature(const char *name, const QByteArrayList &parameterTypes,
+                            const char *returnType)
+{
+    QByteArray result;
+    if (isNonVoidReturn(returnType))
+        result += QByteArray(returnType) + ' ';
+    result += QByteArray(name) + '(' + parameterTypes.join(", ") + ')';
+    return result;
+}
+
+int SignalManager::callPythonMetaMethod(const QByteArrayList &parameterTypes,
+                                        const char *returnType,
+                                        void **args, PyObject *callable)
+{
+    Q_ASSERT(callable);
+
+    Shiboken::GilState gil;
+    int callResult = callPythonMetaMethodHelper(parameterTypes, returnType, args, callable);
+    switch (callResult) {
+    case CallOk:
+        return 0;
+    case CallOtherError:
+        return -1;
+    case CallReturnValueError: {
+        const auto &sig = signature("slot", parameterTypes, returnType);
+        PyErr_SetString(PyExc_RuntimeError, msgCannotConvertReturn(sig).constData());
+        return -1;
+    }
+    default: { // CallArgumentError + n
+        const int arg = callResult - CallArgumentError;
+        const auto &sig = signature("slot", parameterTypes, returnType);
+        const auto &msg = msgCannotConvertParameter(parameterTypes.at(arg), sig, arg);
+        PyErr_SetString(PyExc_TypeError, msg.constData());
+        return -1;
+    }
+    }
+    return 0;
 }
 
 bool SignalManager::registerMetaMethod(QObject *source, const char *signature, QMetaMethod::MethodType type)
@@ -610,45 +631,71 @@ QDebug operator<<(QDebug debug, const slotSignature &sig)
     return debug;
 }
 
-int SignalManager::registerMetaMethodGetIndex(QObject *source, const char *signature, QMetaMethod::MethodType type)
+static int addMetaMethod(QObject *source, const QByteArray &signature,
+                         QMetaMethod::MethodType type)
 {
-    if (!source) {
-        qWarning("SignalManager::registerMetaMethodGetIndex(\"%s\") called with source=nullptr.",
-                 signature);
+    const QMetaObject *metaObject = source->metaObject();
+    SbkObject *self = Shiboken::BindingManager::instance().retrieveWrapper(source);
+    if (!Shiboken::Object::hasCppWrapper(self)) {
+        qWarning().noquote().nospace() << __FUNCTION__
+            << ": Cannot add dynamic method \"" << signature << "\" (" << type
+            << ") to " << source << ": No Wrapper found.";
+        return -1;
+    }
+
+    auto *pySelf = reinterpret_cast<PyObject *>(self);
+    auto *dict = SbkObject_GetDict_NoRef(pySelf);
+    MetaObjectBuilder *dmo = metaBuilderFromDict(dict);
+    // Create a instance meta object
+    if (dmo == nullptr) {
+        dmo = new MetaObjectBuilder(Py_TYPE(pySelf), metaObject);
+        PyObject *pyDmo = PyCapsule_New(dmo, nullptr, destroyMetaObject);
+        PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
+        Py_DECREF(pyDmo);
+    }
+
+    if (type == QMetaMethod::Slot) {
+        qCWarning(lcPySide).noquote().nospace()
+            << "Warning: Registering dynamic slot \""
+            << signature << "\" on \"" << source->metaObject()->className()
+            << "\". Consider annotating with " << slotSignature(signature);
+    }
+
+    return type == QMetaMethod::Signal ? dmo->addSignal(signature) : dmo->addSlot(signature);
+}
+
+static inline void warnNullSource(const char *signature)
+{
+    qWarning("SignalManager::registerMetaMethodGetIndex(\"%s\") called with source=nullptr.",
+             signature);
+}
+
+int SignalManager::registerMetaMethodGetIndex(QObject *source, const char *signature,
+                                              QMetaMethod::MethodType type)
+{
+    if (source == nullptr) {
+        warnNullSource(signature);
         return -1;
     }
     const QMetaObject *metaObject = source->metaObject();
-    int methodIndex = metaObject->indexOfMethod(signature);
-    // Create the dynamic signal is needed
-    if (methodIndex == -1) {
-        SbkObject *self = Shiboken::BindingManager::instance().retrieveWrapper(source);
-        if (!Shiboken::Object::hasCppWrapper(self)) {
-            qWarning() << "Invalid Signal signature:" << signature;
-            return -1;
-        }
-        auto *pySelf = reinterpret_cast<PyObject *>(self);
-        auto *dict = SbkObject_GetDict_NoRef(pySelf);
-        MetaObjectBuilder *dmo = metaBuilderFromDict(dict);
+    const int methodIndex = metaObject->indexOfMethod(signature);
+    // Create the dynamic signal if needed
+    return methodIndex != -1
+        ? methodIndex : addMetaMethod(source, QByteArray(signature), type);
+}
 
-        // Create a instance meta object
-        if (!dmo) {
-            dmo = new MetaObjectBuilder(Py_TYPE(pySelf), metaObject);
-            PyObject *pyDmo = PyCapsule_New(dmo, nullptr, destroyMetaObject);
-            PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
-            Py_DECREF(pyDmo);
-        }
-
-        if (type == QMetaMethod::Slot) {
-            qCWarning(lcPySide).noquote().nospace()
-                << "Warning: Registering dynamic slot \""
-                << signature << "\" on " << source->metaObject()->className()
-                << ". Consider annotating with " << slotSignature(signature);
-        }
-
-        return type == QMetaMethod::Signal
-            ? dmo->addSignal(signature) : dmo->addSlot(signature);
+int SignalManager::registerMetaMethodGetIndexBA(QObject* source, const QByteArray &signature,
+                                                QMetaMethod::MethodType type)
+{
+    if (source == nullptr) {
+        warnNullSource(signature.constData());
+        return -1;
     }
-    return methodIndex;
+    const QMetaObject *metaObject = source->metaObject();
+    const int methodIndex = metaObject->indexOfMethod(signature.constData());
+    // Create the dynamic signal if needed
+    return methodIndex != -1
+        ? methodIndex : addMetaMethod(source, signature, type);
 }
 
 const QMetaObject *SignalManager::retrieveMetaObject(PyObject *self)
@@ -670,34 +717,3 @@ const QMetaObject *SignalManager::retrieveMetaObject(PyObject *self)
 
     return builder->update();
 }
-
-namespace {
-
-static PyObject *parseArguments(const QList<QByteArray>& paramTypes, void **args)
-{
-    const qsizetype argsSize = paramTypes.size();
-    PyObject *preparedArgs = PyTuple_New(argsSize);
-
-    for (qsizetype i = 0; i < argsSize; ++i) {
-        void *data = args[i+1];
-        const char *dataType = paramTypes[i].constData();
-        Shiboken::Conversions::SpecificConverter converter(dataType);
-        if (converter) {
-            PyTuple_SET_ITEM(preparedArgs, i, converter.toPython(data));
-        } else {
-            PyErr_Format(PyExc_TypeError, "Can't call meta function because I have no idea how to handle %s", dataType);
-            Py_DECREF(preparedArgs);
-            return nullptr;
-        }
-    }
-    return preparedArgs;
-}
-
-static bool emitShortCircuitSignal(QObject *source, int signalIndex, PyObject *args)
-{
-    void *signalArgs[2] = {nullptr, args};
-    source->qt_metacall(QMetaObject::InvokeMetaMethod, signalIndex, signalArgs);
-    return true;
-}
-
-} //namespace
